/*****************************************************************************
  Copyright  1998 - 2003 by Martin Cook. All rights are reserved. If you like 
  this code then feel free to go ahead and use it. The only thing I ask is 
  that you don't remove or alter my copyright notice. Your use of this 
  software is entirely at your own risk. I make no claims or warrantees about 
  the reliability or fitness of this code for any particular purpose. If you 
  make changes or additions to this code please mark your code as being 
  yours. If you have questions or comments then please contact me at: 
  martinc@outdrs.net
  
  Have Fun! :o)
*****************************************************************************/

#include "stdafx.h"
#include "StatusBarEx.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif // _DEBUG

// ***************************************************************************
// CStatusBarChildWnd definition.
// ***************************************************************************

class CStatusBarChildWnd : public CWnd
{
public:

	CStatusBarChildWnd( int nMaxWidth )
	{
		m_nMaxWidth = nMaxWidth;
	} // End CStatusBarChildWnd()

	inline int GetMaxWidth( void ) const
	{
		return m_nMaxWidth;
	} // End GetMaxWidth()

private:

	int m_nMaxWidth;

};

// ***************************************************************************
// CStatusBarEx maps and macros.
// ***************************************************************************

BEGIN_MESSAGE_MAP(CStatusBarEx, CStatusBar)
	//{{AFX_MSG_MAP(CStatusBarEx)
	ON_WM_SIZE()
	ON_WM_DESTROY()
	ON_WM_CREATE()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

// ***************************************************************************
// CStatusBarEx implementation.
// ***************************************************************************

CStatusBarEx::CStatusBarEx( void )
{
	
	// Ensure that ActiveX container support is installed.
	AfxEnableControlContainer();

	ASSERT_VALID( this );

} // End CStatusBarEx()

// ***************************************************************************

CStatusBarEx::~CStatusBarEx( void )
{
	ASSERT_VALID( this );
} // End ~CStatusBarEx()

// ***************************************************************************

int CStatusBarEx::OnCreate( LPCREATESTRUCT lpCreateStruct ) 
{

	ASSERT_VALID( this );

	// Create the status bar.
	if ( -1 == CStatusBar::OnCreate( lpCreateStruct ) )
		return -1;
	
	// Ensure a status area is shown by default.
	UINT ind[] = { ID_SEPARATOR };
	SetIndicators( ind, sizeof( ind ) / sizeof( ind[ 0 ] ) );
	
	return 0;

} // End OnCreate()

// ***************************************************************************

void CStatusBarEx::OnDestroy( void ) 
{

	ASSERT_VALID( this );

	// Destroy the status bar.
	CStatusBar::OnDestroy();

	// Attempt to get an iterator for the map.
	POSITION pos = m_mapChildren.GetStartPosition();

	UINT nID = 0;
	CStatusBarChildWnd* pWnd = NULL;

	// Loop and destroy everything in the map.
	while ( NULL != pos )
	{

		// Get the next child window from the map.
		m_mapChildren.GetNextAssoc( pos, nID, pWnd );
		ASSERT_VALID( pWnd );

		// Destroy the window element and wrapper object.
		pWnd->DestroyWindow();
		::delete pWnd;

	} // End while there are more child windows in the map.

	// Empty the map.
	m_mapChildren.RemoveAll();
			
} // End OnDestroy()

// ***************************************************************************

void CStatusBarEx::OnSize( UINT nType, int cx, int cy )  
{
	
	ASSERT_VALID( this );

	// Resize the status bar.
	CStatusBar::OnSize( nType, cx, cy );

	// Resize any child windows.
	_ResizeChildWindows();
	
} // End OnSize()

// ***************************************************************************

BOOL CStatusBarEx::AddChildWindow( LPCTSTR lpszClass, UINT nID, int nWidth,
	DWORD dwStyle )
{

	ASSERT_VALID( this );
	ASSERT( NULL != GetSafeHwnd() );
		
	CStatusBarChildWnd* pWnd = NULL;

	// Verify that the specified child window isn't already in the map.
	if ( TRUE == m_mapChildren.Lookup( nID, pWnd ) )
		return FALSE;

	try
	{

		// Create a new child window wrapper object.
		pWnd = ::new CStatusBarChildWnd( nWidth );

		// Attempt to create a window element for the wrapper object.
		if ( FALSE == pWnd->CreateControl( lpszClass, NULL, 
			dwStyle | (WS_CHILD | WS_VISIBLE), CRect( 0, 0, 0, nWidth ), 
			this, nID ) )
			AfxThrowUserException();

		// Add the child window to the map.
		m_mapChildren[ nID ] = pWnd;

		// Rebuild the panes for the status bar.
		return _RebuildPanes();

	} // End try

	catch ( CException* e )
	{

		e->Delete();

		// Cleanup the window wrapper.
		::delete pWnd;

		return FALSE;

	} // End catch

} // End AddChildWindow()

// ***************************************************************************

BOOL CStatusBarEx::RemoveChildWindow( UINT nID )
{

	ASSERT_VALID( this );
	ASSERT( NULL != GetSafeHwnd() );

	CStatusBarChildWnd* pWnd = NULL;

	// Attempt to locate the specified child window in the map.
	if ( FALSE == m_mapChildren.Lookup( nID, pWnd ) )
		return FALSE;

	ASSERT_VALID( pWnd );

	// Destroy the window element.
	pWnd->DestroyWindow();
	
	// Cleanup the wrapper object.
	::delete pWnd;

	// Attempt to remove the child window from the map.
	VERIFY( TRUE == m_mapChildren.RemoveKey( nID ) );

	// Rebuild the panes for the status bar.
	return _RebuildPanes();

} // End RemoveChildWindow()

// ***************************************************************************

CWnd* CStatusBarEx::GetChildWindow( UINT nID ) const
{
	
	ASSERT_VALID( this );

	// Attempt to locate the specified child window in the map.
	CStatusBarChildWnd* pWnd = NULL;
	m_mapChildren.Lookup( nID, pWnd );

	return pWnd;

} // End GetChildWindow()

// ***************************************************************************

BOOL CStatusBarEx::_RebuildPanes( void )
{

	ASSERT_VALID( this );
	ASSERT( NULL != GetSafeHwnd() );

	UINT* pIndicators = NULL;

	try
	{

		// Get a count of all the child windows in the map - adding 
		//   one to account for the status area.
		int nCount = m_mapChildren.GetCount() + 1;
		
		// Create a buffer to hold dummy identifiers. Note: we set all
		//   the identifers to ID_SEPARATOR here to prevent MFC from 
		//   automatically searching for strings to associate with panes
		//   in the status bar - separators don't have strings by default.
		pIndicators = ::new UINT[ nCount ];
		::memset( pIndicators, ID_SEPARATOR, sizeof( UINT ) * nCount );

		// Attempt to update the status bar.
		if ( FALSE == SetIndicators( pIndicators, nCount ) )
			AfxThrowUserException();

		// Cleanup the buffer.
		::delete[] pIndicators;
		pIndicators = NULL;
		
		// Attempt to get an iterator for the map.
		POSITION pos = m_mapChildren.GetStartPosition();
		UINT nIndex = 1;

		UINT nID = 0;
		CStatusBarChildWnd* pWnd = NULL;

		// At this point, we must modify all the panes that correspond
		//   to child windows by forcing each one to contain the proper 
		//   width, style, and identifier. 

		while ( NULL != pos )
		{

			// Attempt to get the next child window from the map.
			m_mapChildren.GetNextAssoc( pos, nID, pWnd );
			ASSERT_VALID( pWnd );

			// Update the properties for this pane.
			SetPaneInfo( nIndex, nID, SBPS_NOBORDERS, 
				pWnd->GetMaxWidth() );

			// Point to the next pane.
			nIndex++;

		} // End while there are more child windows in the map.

		// Resize all the child windows.
		return _ResizeChildWindows();

	} // End try

	catch ( CException* e )
	{
		
		e->Delete();

		// Cleanup the buffer.
		::delete[] pIndicators;

		return FALSE;

	} // End catch

} // End _RebuildPanes()

// ***************************************************************************

BOOL CStatusBarEx::_ResizeChildWindows( void )
{

	ASSERT_VALID( this );
	ASSERT( NULL != GetSafeHwnd() );

	// Get a count of all the child windows in the map.
	int nCount = m_mapChildren.GetCount();

	// Are there any windows to resize?
	if ( 0 == nCount )
		return TRUE;

	// Attempt to get an iterator for the map.
	POSITION pos = m_mapChildren.GetStartPosition();
	UINT nIndex = 1;

	UINT nID = 0;
	CStatusBarChildWnd* pWnd = NULL;

	// Attempt to begin a deferred window position operation. 
	HDWP hDwp = ::BeginDeferWindowPos( nCount );
	ASSERT( NULL != hDwp );
	
	// Loop and resize child windows.
	while ( NULL != pos )
	{

		// Attempt to get the next child window from the map.
		m_mapChildren.GetNextAssoc( pos, nID, pWnd );
		ASSERT_VALID( pWnd );

		// Calculate a footprint for child window by using the
		//   footprint of the underlying status bar pane.
		CRect rc;
		GetItemRect( nIndex, &rc );

		// Perform a deferred move operation.
		::DeferWindowPos( hDwp, pWnd->GetSafeHwnd(), 
			GetSafeHwnd(), rc.left, rc.top, rc.Width(), rc.Height(), 
			SWP_NOZORDER );

		// Point to the next pane.
		nIndex++;

	} // End while there are more child windows in the map.

	// End the deferred window operation.
	::EndDeferWindowPos( hDwp );
		
	return TRUE;	

} // End ResizeChildWindows()

// ********************************* EOF *************************************